#!/usr/bin/perl
# fixup-wrapdescriptions.pl
# Word-wrap the DESCRIPTION clauses of a MIB.  

use English; 
use newtrace;
use strict; 

defined($ARGV[1-1]) || die "Usage: fixup  mib-file\n";

# Now the MIB file
my $sMibFile = $ARGV[1-1];
newtrace::trace( 1,"file1|$sMibFile|");
open( MIB,  '<', $sMibFile ) || die "Error: cannot open input file $sMibFile: $!";

my $nFurtherIndent = 2; # how far to indent beyond the DESCRIPTION keyword
my $nRightMargin = 66;  # max line length, right margin, including all indents

my $sInput = "";
my $sOutput = "";
my $sNewContent = "";
my ($sLeft,$sBigMid,$sMidBegin,$sMidContent,$sMidEnd,$sRight);
my ($nCurrentIndent,$nFullIndent);
my ($nCurLen,$sNextWord,$nNextWord);

# Get the entire file into a single string.
my $sInput = "";
while (<MIB>)
{
    $sInput .= $_;
}

# Find each DESCRIPTION string and isolate its contents.
# Replace it, then move on to the next DESCRIPTION, if any.  
my $sOutput = "";
while ( $sInput ) 
{
    ($sLeft,$sBigMid,$sMidBegin,$sMidContent,$sMidEnd,$sRight) = $sInput =~ 
        /^(.*?)(([ \t]*DESCRIPTION\s*\")([^\"]*)(\"\s*\n))(.*)$/ms;
    newtrace::trace(3, "Left|$sLeft|");
    newtrace::trace(2, "Mid|$sMidContent|");
    newtrace::trace(2, "BigMid|$sBigMid|");
    #newtrace::trace(3, "Right|$sRight|");

    # If there is a DESCRIPTION, wrap the content within 
    # reasonable margins.  
    if ( $sLeft ) 
    {
        $nCurrentIndent = fniGetIndent( $sMidBegin );
        $nFullIndent = $nCurrentIndent + $nFurtherIndent;
        $sNewContent = fnsRewrap( $sMidContent, $nFullIndent, $nRightMargin );
        # Fixup the wrapped version: remove intro blanks and trailing newline
        $sNewContent =~ s/^[ ]*//;
        chomp $sNewContent;

        # Append the left portion to the output string, 
        # then the DESCRIPTION keyword and white space, 
        # then the re-formed content and the closing quotes.
        $sOutput .= $sLeft;
        $sOutput .= $sMidBegin;
        $sOutput .= "$sNewContent";  
        $sOutput .= $sMidEnd;

        # And whatever was left on the right becomes 
        # the new input to be searched.  
        $sInput = $sRight;
    } else
    {
        $sOutput .= $sInput;    # last stuff in the buffer
        $sInput = "";
    }
    #newtrace::trace(3, "OUTPUT|$sOutput|");
}

# Print the output and we're done.  
print "$sOutput\n";

#  f n s R e w r a p 
# Word-wrap the input string between the stated margins.
sub fnsRewrap 
{
    my($sUnwrapped,$nLeft,$nRight) = @_;
    newtrace::trace(1,"entr fnsRewrap str|$sUnwrapped| lm|$nLeft| rm|$nRight|");
    my($sWrapped) = "";
    $sUnwrapped =~ s/\n */ /g;
    $sUnwrapped =~ s/\r//g;

    my @lWords = ();
    @lWords = split( / /, $sUnwrapped );
    my $sLeftIndent = sprintf( "%*s", $nLeft, "" );
    my $nLineLen = $nRight - $nLeft;
    my $sLine = "";
    my $nWords = @lWords;
    newtrace::trace(3,"proc nwords|$nWords| linelen|$nLineLen|");
    while ( @lWords )
    {
        LINE: while ( length( $sLine ) < $nLineLen )
        {
            $nCurLen = length( $sLine );
            my $nRemainingWords = @lWords;
            last LINE if ( !$nRemainingWords );
            $sNextWord = shift( @lWords );
            $nNextWord = length( $sNextWord );
            newtrace::trace(4,"proc gotword word|$sNextWord|$nNextWord| line|$sLine|$nCurLen|");
            if ( ($nCurLen + $nNextWord) < $nLineLen )
            {
                $sLine .= " " if ($sLine);
                $sLine .= $sNextWord;
            } else
            {
                unshift( @lWords, $sNextWord );
                last LINE; 
            }
        }
        $nCurLen = length( $sLine );
        newtrace::trace(3,"proc gotline line|$sLine|$nCurLen|");
        $sWrapped .= $sLeftIndent . $sLine . "\n";
        $sLine = "";
    }
    
    newtrace::trace(2,"exit fnsRewrap wrapped|\n$sWrapped|");
    return( $sWrapped );
    
}

#  f n s G e t I n d e n t 
# Find out how far the string should be indented.
sub fniGetIndent
{
    my($sKeyword) = @_;
    $sKeyword =~ s/\t/    /g;
    my ($sBlanks) = $sKeyword =~ /^([ ]*)/;
    my $nBlanks = length( $sBlanks );
    newtrace::trace(2,"exit fnGetIndent str|$sKeyword| indent|$sBlanks| len|$nBlanks|");
    return $nBlanks;
}
